import MDAnalysis as mda
from MDAnalysis.analysis import rdf
import numpy as np
import click
from scipy import integrate
from mdkits.util import arg_type


def calculate_Gab(r_values, gab_values):
    dr = r_values[1] - r_values[0]

    integrand = 4 * np.pi * r_values**2 * gab_values

    G_ab = np.cumsum(integrand) * dr

    return G_ab


@click.command(name="rdf")
@click.argument("filename", type=click.Path(exists=True))
@click.option('--cell', type=arg_type.Cell, help='set cell, a list of lattice, --cell x,y,z or x,y,z,a,b,c')
@click.option("--group", type=click.Tuple([str, str]), help="two group to analysis")
@click.option('--range', type=click.Tuple([float, float]), help="the range of rdf")
@click.option('-r', type=arg_type.FrameRange, help='range of frame to analysis')
def main(filename, cell, group, range, r):
    """analysis the radial distribution function"""
    u = mda.Universe(filename)
    u.trajectory.ts.dt = 0.0001
    u.dimensions = cell
    o = f"rdf_{'_'.join(group).replace(' ', '_')}.dat"

    rho = 32/(9.86**3)

    group1 = u.select_atoms(group[0])
    group2 = u.select_atoms(group[1])

    crdf = rdf.InterRDF(group1, group2, verbose=True, range=(range[0], range[1]), norm='rdf')


    if r is not None:
        if len(r) == 2:
            crdf.run(start=r[0], stop=r[1])
        elif len(r) == 3:
            crdf.run(start=r[0], stop=r[1], step=r[2])
    else:
        crdf.run()

    combin = np.column_stack((crdf.results.bins, crdf.results.rdf, calculate_Gab(crdf.results.bins, crdf.results.rdf)*rho))
    np.savetxt(o, combin, header="A\tgr\tNr", fmt="%.5f", delimiter='\t')


if __name__ == "__main__":
    main()


#!/usr/bin/env python3

import os
from MDAnalysis import Universe
import MDAnalysis, click
from mdkits.util import (
    arg_type,
    os_operation,
    cp2k_input_parsing
    )


@click.command(name='wrap')
@click.argument('filename', type=click.Path(exists=True), default=os_operation.default_file_name('*-pos-1.xyz', last=True))
@click.option('-o', type=str, help='output file name', default='wraped.xyz', show_default=True)
@click.option('--cell', type=arg_type.Cell, help='set cell from cp2k input file or a list of lattice: --cell x,y,z or x,y,z,a,b,c', default='input.inp', show_default=True)
def main(filename, o, cell):
    """
    wrap the coordinates in a cell from a trajectory file
    """

    u = Universe(filename)
    u.dimensions = cell
    ag = u.select_atoms("all")

    with MDAnalysis.Writer(o, ag.n_atoms) as W:
        for ts in u.trajectory:
            ag.wrap()
            W.write(ag)

    click.echo(f"\nwrap is done, output file {o} is:")
    click.echo(os.path.abspath(o))


if __name__ == '__main__':
    main()



import numpy as np
import click
import MDAnalysis
from MDAnalysis import Universe
from MDAnalysis.analysis.base import AnalysisBase
from mdkits.util import (
    arg_type,
    numpy_geo,
    encapsulated_mda,
    os_operation
)
from .setting import common_setting
import sys


class Dipole_distribution(AnalysisBase):
    def __init__(self, filename, cell, update_water, distance_judg, angle_judg, surface, dt=0.001, bin_size=0.2):
        u = Universe(filename)
        u.trajectory.ts.dt = dt
        u.dimensions = cell

        self.u = u
        self.atomgroup = u.select_atoms("all")
        self.bin_size = bin_size
        self.frame_count = 0
        self.surface = surface
        self.update_water = update_water
        self.mid_z = u.dimensions[2] / 2

        if self.update_water:
            self.distance_judg = distance_judg
            self.angle_judg = angle_judg

        if surface is not None:
            self.surface_group = self.atomgroup.select_atoms(f"{surface}")
            if self.surface_group.n_atoms == 0:
                sys.exit("Please specify the correct surface group")
        else:
            self.surface_group = False

        super(Dipole_distribution, self).__init__(self.atomgroup.universe.trajectory, verbose=True)

    def _prepare(self):
        self.bin_num = int(self.u.dimensions[2] / self.bin_size) + 2
        self.dipole_distribution = np.zeros(self.bin_num, dtype=np.float64)
        self.o_count = np.zeros(self.bin_num, dtype=np.float64)

        if self.surface_group:
            self.surface_pos = np.zeros(2)

    def _append(self, angle, z):
        bins = np.floor(z / self.bin_size).astype(int) + 1
        np.add.at(self.dipole_distribution, bins, angle)
        np.add.at(self.o_count, bins , 1)

    def _single_frame(self):
        o_group = self.atomgroup.select_atoms("name O")

        if self.update_water:
            h_group = self.atomgroup.select_atoms("name H")
            o, oh1, oh2 = encapsulated_mda.update_water(self, o_group, h_group, distance_judg=self.distance_judg, angle_judg=self.angle_judg, return_index=False)
        else:
            o = o_group
            oh1 = self.atomgroup[o.indices + 1]
            oh2 = self.atomgroup[o.indices + 2]

        if self.surface_group:
            lower_z, upper_z = numpy_geo.find_surface(self.surface_group.positions[:, 2])
            self.surface_pos[0] += lower_z
            self.surface_pos[1] += upper_z

        vec1 = MDAnalysis.lib.distances.minimize_vectors(oh1.positions - o.positions, self.u.dimensions)
        vec2 = MDAnalysis.lib.distances.minimize_vectors(oh2.positions - o.positions, self.u.dimensions)

        bisector = numpy_geo.vector_between_two_vector(vec1, vec2)


        angle_bisector = np.hstack((bisector[o.positions[:, 2] < self.mid_z][:, 2] / np.linalg.norm(bisector[o.positions[:, 2] < self.mid_z], axis=1), -bisector[o.positions[:, 2] > self.mid_z][:, 2] / np.linalg.norm(bisector[o.positions[:, 2] > self.mid_z], axis=1)))

        self._append(angle_bisector, np.hstack((o.positions[:, 2][o.positions[:, 2] < self.mid_z], o.positions[:, 2][o.positions[:, 2] > self.mid_z])))

        self.frame_count += 1

    def _conclude(self):
        if self.frame_count > 0:
            average_dipole = self.dipole_distribution / self.o_count
            water_density = (self.o_count * (15.999+1.0008*2) * 1.660539 / (self.u.dimensions[0] * self.u.dimensions[1] * self.bin_size)) / self.frame_count
            average_dipole = average_dipole * water_density
            bins_z = np.arange(len(average_dipole)) * self.bin_size + self.bin_size / 2

            if self.surface:
                lower_z = self.surface_pos[0] / self.frame_count
                if self.surface_pos[1] == 0:
                    upper_z = np.inf
                else:
                    upper_z = self.surface_pos[1] / self.frame_count

                mask = (bins_z >= lower_z) & (bins_z <= upper_z)
                filtered_bins_z = bins_z[mask] - lower_z
                filtered_dipole_distribution = average_dipole[mask]

                conbined_data = np.column_stack((filtered_bins_z, filtered_dipole_distribution))
            else:
                conbined_data = np.column_stack((bins_z, average_dipole))

            np.savetxt("dipole_distribution.dat", conbined_data, header="z\tDipole Z Positions\tWater Density", fmt='%.5f', delimiter='\t')


@click.command(name="dipole")
@common_setting
def main(filename, cell, update_water, distance, angle, surface, r):
    """analysis dipole along z-axis"""
    a = Dipole_distribution(filename, cell, update_water, distance_judg=distance, angle_judg=angle, surface=surface)
    if r is not None:
        if len(r) == 2:
            a.run(start=r[0], stop=r[1])
        elif len(r) == 3:
            a.run(start=r[0], stop=r[1], step=r[2])
    else:
        a.run()


if __name__ == "__main__":
    main()


import click
from mdkits.dft_cli import (
    cube,
    pdos,
)


@click.group(name='dft')
@click.pass_context
def main(ctx):
    """kits for dft analysis"""
    pass


main.add_command(cube.main)
main.add_command(pdos.main)

if __name__ == '__main__':
    main()


import numpy as np
import click
import MDAnalysis
from MDAnalysis import Universe
from MDAnalysis.analysis.base import AnalysisBase
from mdkits.util import os_operation, arg_type


class Velocity_AutoCorrelation(AnalysisBase):
    def __init__(self, filename, select, dt=0.001):
        u = Universe(filename)
        u.trajectory.ts.dt = dt
        self.u = u
        self.atomgroup = u.select_atoms(select)

        super(Velocity_AutoCorrelation, self).__init__(self.atomgroup.universe.trajectory, verbose=True)

    def _prepare(self):
        self.cvv = []
        self.v0 = self.atomgroup.positions
        self.normalize = 1/np.sum(self.v0*self.v0)
        self.cvv.append(np.sum(self.v0*self.v0)*self.normalize)


    def _append(self, cvv):
        self.cvv.append(cvv*self.normalize)

    def _single_frame(self):
        cvv = np.sum(self.atomgroup.positions*self.v0)
        self._append(cvv)

    def _conclude(self):
        self.cvv = np.array(self.cvv)

        sf = self.cvv.shape[0]
        fftraj = np.fft.rfft(self.cvv)
        fdos = np.abs(fftraj)

        faxis = np.fft.rfftfreq(sf, d=1/sf)

        combine = np.column_stack((np.arange(len(self.cvv)), self.cvv))

        np.savetxt('vac.dat', combine, fmt='%.5f', header="frame\tvac")
        np.savetxt('freq.dat', np.column_stack((faxis, fdos)), fmt='%.5f', header="freq\tabundance")


@click.command(name="vac")
@click.argument("filename", type=click.Path(exists=True), default=os_operation.default_file_name('*-vel-1.xyz', last=True))
@click.option("--select", type=str, default="all", help="atom selection", show_default=True)
@click.option('-r', type=arg_type.FrameRange, help='range of frame to analysis')
def main(filename, select, r):
    """analysis velocity autocorrelation function and frequency"""
    a = Velocity_AutoCorrelation(filename, select)

    if r is not None:
        if len(r) == 2:
            a.run(start=r[0], stop=r[1])
        elif len(r) == 3:
            a.run(start=r[0], stop=r[1], step=r[2])
    else:
        a.run()


if __name__ == '__main__':
    main()


import click
from mdkits.util import arg_type, out_err


@click.command(name="interface")
@click.option('--slab', type=arg_type.Structure, help='surface')
@click.option('--sol', type=arg_type.Structure, help='solution')
@click.option('--interval', type=float, help='interval between surface and sol', default=2, show_default=True)
@click.option('--cap', type=click.Choice(['ne', 'slab']), help='build slab interface')
@click.option('--vacuum', type=float, help='vacuum length', default=0, show_default=True)
def main(slab, sol, interval, cap, vacuum):
    """build interface"""
    out_err.check_cell(slab)
    out_err.check_cell(sol)

    o = f"{slab.filename.split('.')[-2]}_{sol.filename.split('.')[-2]}.cif"

    slab.set_pbc(True)
    slab.center()
    slab_cell = slab.cell.cellpar()
    init_slab_cell = slab.cell.cellpar()
    if cap == 'slab':
        slab_copy = slab.copy()

    sol_cell = sol.cell.cellpar()
    sol.set_pbc(True)
    sol.center()
    sol.positions[:, 2] += slab_cell[2] + interval

    slab.extend(sol)
    slab_cell[2] += 2 * interval + sol_cell[2]
    slab.set_cell(slab_cell)
    slab.center()

    if cap is None:
        slab.positions[:, 2] -= 0.5 * init_slab_cell[2]
    elif cap == 'ne':
        from ase import Atoms
        ne_interval = 4
        lenx = init_slab_cell[0]
        leny = init_slab_cell[1]
        ne_cell = [lenx, leny, 2, 90, 90, 90]
        ne_position = []
        ne_symbols = []
        ne_site = [int(lenx//ne_interval), int(leny//ne_interval)]
        for i in range(ne_site[0]):
            for j in range(ne_site[1]):
                ne_position.append((i*ne_interval, j*ne_interval, 0))
                ne_symbols.append('Ne')
        ne_atoms = Atoms(symbols=ne_symbols, positions=ne_position, cell=ne_cell)
        ne_atoms.center()

        slab.positions[:, 2] += -(slab_cell[2] + interval)
        slab.extend(ne_atoms)
        slab_cell[2] += ne_cell[2]
        slab.set_cell(slab_cell)
        slab.center()
    elif cap == 'slab':
        slab.positions[:, 2] += -(slab_cell[2] + interval)
        slab.extend(slab_copy)
        slab_cell[2] += slab_copy.cell.cellpar()[2]
        slab.set_cell(slab_cell)
        slab.center()
    
    if vacuum > 0:
        slab_cell[2] += vacuum
        slab.set_cell(slab_cell)


    slab.write(o)
    out_err.path_output(o)

if __name__ == '__main__':
    main()


"""
filename: numpy_geo.py
function: some vector calculation with numpy
"""


from numpy import array, dot, abs, cross
from numpy.linalg import norm
import numpy as np



def point_plane_distance(plane):
# distance of point with a plane(a list with [normal_voctor, a_point_in_plane])
    def point(point):
        return abs(dot(plane[0], point - plane[1])) / norm(plane[0])
    return point


def point_point_distance(point1, point2):
# distance of point with a point
    return norm(point1 - point2)


def point(frame):
# return a point of atom, from .xyz file
    def point_index(index):
        return array(frame[index].split()[1:]).astype(float)
    return point_index

def plane(frame, index1, index2, index3):
# return a normal voctor of a plane with three point(index and group)
    get_point = point(frame)
    base1 = get_point(index1)
    base2 = get_point(index2)
    base3 = get_point(index3)
    return [cross(base2 - base1, base3 - base1), base3]


def vector_between_two_vector(vector1, vector2):
    vector1_unit = vector1 / np.linalg.norm(vector1)
    vector2_unit = vector2 / np.linalg.norm(vector2)
    vector = vector1_unit + vector2_unit
    return vector


def vector_vector_angle(vector, surface_vector):
    if len(vector.shape) == 1:
        cos = np.dot(vector, surface_vector) / (np.linalg.norm(vector) * np.linalg.norm(surface_vector))
    else:
        cos = np.dot(vector, surface_vector) / (np.linalg.norm(vector, axis=1) * np.linalg.norm(surface_vector))
    vector_vector_angle = np.arccos(np.clip(cos, -1.0, 1.0))
    vector_vector_angle = np.degrees(vector_vector_angle)
    return vector_vector_angle


def cell_to_wrap_coefficients(cell, z=False):
    angle = np.radians(180-cell[-1])
    dangle = [np.cos(angle), np.sin(angle)]
    xyz = cell[0:3]
    if z:
        coefficients = np.array([
            [(kx * xyz[0] + ky * dangle[0] * xyz[1]), (ky * dangle[1] * xyz[1]), kz * xyz[2]]
            for kx in range(-1, 2)
            for ky in range(-1, 2)
            for kz in range(-1, 2)
        ])
    else:
        coefficients = np.array([
            [(kx * xyz[0] + ky * dangle[0] * xyz[1]), (ky * dangle[1] * xyz[1]), 0]
            for kx in range(-1, 2)
            for ky in range(-1, 2)
        ])

    return coefficients


def unwrap(atom1, atom2, coefficients, max=0, total=False):
    init_dist = point_point_distance(atom1, atom2)
    if total:
        atoms = atom2 + coefficients
        distance = np.linalg.norm(atoms-atom1, axis=1)

        return distance, atoms
    else:
        if init_dist > max:
            min_dist = float('inf')
            closest_point = None
            atoms = atom2 + coefficients
            distance = np.linalg.norm(atoms-atom1, axis=1)
            min_index = np.argmin(distance)
            min_dist = distance[min_index]
            closest_point = atoms[min_index]

        else:
            closest_point = atom2
            min_dist = init_dist

        return min_dist, closest_point


def find_surface(surface_group:np.ndarray, layer_tolerance=0.05, surface_tolerance=5):
    sort_group = np.sort(surface_group)
    layer_mean = []
    current_layer = [sort_group[0]]
    for i in range(1, len(sort_group)):
        if abs(sort_group[i] - sort_group[i-1]) < layer_tolerance:
            current_layer.append(sort_group[i])
        else:
            layer_mean.append(np.mean(current_layer))
            current_layer = [sort_group[i]]
        layer_mean.append(np.mean(current_layer))
    
    if len(layer_mean) == 1:
        return [layer_mean[0], 0]

    diff = np.diff(layer_mean)
    if np.any(diff > surface_tolerance):
        index = np.argmax(diff > surface_tolerance)
        return (layer_mean[index], layer_mean[index + 1])
    else:
        if layer_mean[-1] > layer_mean[0]:
            return [layer_mean[-1], 0]
        return (layer_mean[-1], layer_mean[0])



#!/usr/bin/env python3

from ase.io import read
import click, os
import numpy as np
from ase.build import make_supercell
from mdkits.util import (
    structure_parsing,
    encapsulated_ase,
    cp2k_input_parsing,
    arg_type,
    out_err,
    )


def supercell(atom, x, y, z):
    P = [ [x, 0, 0], [0, y, 0], [0, 0, z] ]
    super_atom = make_supercell(atom, P)
    return super_atom


@click.command(name='supercell')
@click.argument('atoms', type=arg_type.Structure)
@click.argument('super', type=click.Tuple([int, int, int]), default=(1, 1, 1))
@click.option('--cell', type=arg_type.Cell, help='set cell, a list of lattice, --cell x,y,z or x,y,z,a,b,c')
def main(atoms, super, cell):
    """make a supercell"""

    out_err.check_cell(atoms, cell)

    atoms.set_pbc(True)
    atoms.wrap()
    super_atom = supercell(atoms, super[0], super[1], super[2])

    o = f"{atoms.filename.split('.')[-2]}_{super[0]}{super[1]}{super[2]}.cif"
    super_atom.write(o)
    print(os.path.abspath(o))


if __name__ == '__main__':
    main()



#!/usr/bin/env python3

from cp2kdata import Cp2kPdos
import click
import numpy as np
from mdkits.util import os_operation


# set argument
@click.command(name='pdos')
@click.argument('filename', type=list, default=os_operation.default_file_name('*-k*.pdos'))
@click.option('-t', '--type', type=str, default='total', show_default=True)
@click.option('-c', '--clos', type=tuple)
def main(filename, type, clos):
    """analysis cp2k pdos file"""
    if type == 'total':
        dos_obj = Cp2kPdos(filename[0])
        dos, ener = dos_obj.get_raw_dos(dos_type=type)
        print(f"analysis of total dos is done")
        np.savetxt('total.pdos', np.column_stack((ener, dos)), header='energy\tdos')
    else:
        if type:
            for file in filename:
                dos_obj = Cp2kPdos(file)
                dos, ener = dos_obj.get_raw_dos(dos_type=type)
                print(f"analysis of {file}'s {type} dos is done")
                np.savetxt(f'{dos_obj.read_dos_element()}_{type}.pdos', np.column_stack((ener, dos)), header='energy\tdos')

        if clos:
            for file in filename:
                dos_obj = Cp2kPdos(file)
                dos, ener = dos_obj.get_raw_dos(usecols=clos)
                print(f"analysis of {file}'s {type} dos is done")
                np.savetxt(f'{dos_obj.read_dos_element()}_{"_".join(clos)}.pdos', np.column_stack((ener, dos)), header='energy\tdos')


if __name__ == '__main__':
    main()


import click, os
from mdkits.util import arg_type
from importlib import resources
from mdkits.cli import convert
import tempfile, platform, subprocess, sys


@click.command(name="solution")
@click.argument("filename", type=click.Path(exists=True), nargs=-1)
@click.option('--infile', is_flag=True, help="read input mode")
@click.option('--install_julia', is_flag=True, help="install julia for system")
@click.option('--install_packmol', is_flag=True, help="init julia install packmol")
@click.option('--water_number', type=int, help="number of water molecules", default=0, show_default=True)
@click.option('-n', type=int, multiple=True, help="number of molecules")
@click.option('--tolerance', type=float, help="tolerance of solution", default=3.5, show_default=True)
@click.option('--cell', type=arg_type.Cell, help="set cell, a list of lattice: --cell x,y,z or x,y,z,a,b,c")
@click.option('--gap', type=float, help="gap between solution and cell", default=1, show_default=True)
def main(filename, infile, install_julia, install_packmol, water_number, n, tolerance, cell, gap):
    """
    build solution model
    """
    os_name = platform.system()

    if install_julia:
        if os_name in ("Darwin", "Linux", "FreeBSD"):
            try:
                subprocess.run(
                ["curl -fsSL https://install.julialang.org | sh"],
                shell=True,  
                check=True,
            )
                print("Julia installed (macOS, Linux, FreeBSD).")
                sys.exit(1)
            except subprocess.CalledProcessError as e:
                print(f"Julia install failed: {e}")
                sys.exit(0)
        elif os_name == "Windows":
            try:
                subprocess.run(
                    ["winget", "install", "--name", "Julia", "--id", "9NJNWW8PVKMN", "-e", "-s", "msstore"],
                    check=True,
                )
                print("Julia installed (Windows).")
                sys.exit(1)
            except subprocess.CalledProcessError as e:
                print(f"Julia install failed: {e}")
                sys.exit(0)
        else:
            print(f"unsupport os: {os_name}")
            sys.exit(0)

    if install_packmol:
        import julia
        julia.install()

        from julia.api import Julia
        jl = Julia(compiled_modules=False)

        from julia import Pkg, Main
        Pkg.add("PyCall")
        Pkg.activate("Packmol", shared=True)
        Pkg.add("Packmol")
        Main.exit()
    else:
        if os_name in ("Darwin", "Linux"):
            from julia.api import Julia
            jl = Julia(compiled_modules=False)

        from julia import Pkg, Main

        if cell is None:
            raise ValueError("cell should be provided")

        if len(filename) == 0 and water_number == 0:
            raise ValueError("at least one file should be provided, or water_number should be greater than 0")

        if os_name in ("Darwin", "Linux"):
            Pkg.activate("Packmol", shared=True)

        while True:
            try:
                Main.using("Packmol")
                break
            except Exception:
                pass

        if infile:
            for file in filename:
                Main.run_packmol(file)
        else:
            if len(n) != len(filename):
                raise ValueError("number of -n should be equal to number of files")

            temp_file = tempfile.NamedTemporaryFile(mode='w+t', delete=False)
            backslash = "\\"

            structure_input = {}
            output_filenames = []
            if len(filename) > 0:
                for index, file in enumerate(filename):
                    structure_input[file] = f"structure {os.path.join(os.getcwd(), file.replace(backslash, '/').replace('./', ''))}\n  number {n[index]}\n  inside box {gap} {gap} {gap} {cell[0]-gap} {cell[1]-gap} {cell[2]-gap}\nend structure\n"

                    output_filenames.append(f"{file.replace(backslash, '/').split('.')[-2].split('/')[-1]}_{n[index]}")

            if water_number > 0:
                water_path = resources.files('mdkits.build_cli').joinpath('water.xyz')

                structure_input["water"] = f"structure {water_path}\n  number {water_number}\n  inside box {gap} {gap} {gap} {cell[0]-gap} {cell[1]-gap} {cell[2]-gap}\nend structure\n"

                output_filenames.append(f"{str(water_path).replace(backslash, '/').split('.')[-2].split('/')[-1]}_{water_number}")

            output_filename = "-".join(output_filenames) + ".xyz"
            head_input = f"tolerance {tolerance}\nfiletype xyz\noutput {os.path.join(os.getcwd(), output_filename)}\npbc {cell[0]} {cell[1]} {cell[2]}\n"

            total_input = head_input + "\n".join(structure_input.values())
    
            temp_file.write(total_input)
            temp_file.flush()
            temp_file.close()

            Main.run_packmol(temp_file.name)

            if os.path.exists(temp_file.name):
                os.remove(temp_file.name)

        print("="*15)
        print(total_input)
        print("="*15)
        convert.main([output_filename, "-c", "--cell", ",".join([str(a) for a in cell])], standalone_mode=False)
        Main.exit()


if __name__ == "__main__":
    main()


#!/usr/bin/env python3

from ase import build
import click, os
from mdkits.util import arg_type, out_err
from mdkits.build_cli import supercell
import numpy as np
import ase.visualize


def find_vector(atoms):
    position = atoms.positions
    max_z = np.max(position[:, 2])
    highest_points = position[position[:, 2] == max_z]

    shortest_vector = None
    min_distance = float('inf')
    x_axis = np.array([1, 0, 0])

    for i in range(len(highest_points)):
        for j in range(i + 1, len(highest_points)):
            point1 = highest_points[i]
            point2 = highest_points[j]
            vector = point2 - point1

            cos_angle = np.dot(vector, x_axis) / (np.linalg.norm(vector) * np.linalg.norm(x_axis))
            angle = np.arccos(cos_angle)

            if angle <= np.pi / 2:
                distance = np.linalg.norm(vector)
                if distance < min_distance:
                    min_distance = distance
                    shortest_vector = vector

    return shortest_vector


@click.command(name='cut')
@click.argument('atoms', type=arg_type.Structure)
@click.option('--face', type=click.Tuple([int, int, int]), help='face index')
@click.option('--size', type=click.Tuple([int, int, int]), help='surface size')
@click.option('--vacuum', type=float, help='designate vacuum of surface, default is None', default=0.0, show_default=True)
@click.option('--cell', type=arg_type.Cell, help='set xyz file cell, --cell x,y,z,a,b,c')
@click.option('--orth', is_flag=True, help='orthogonalize cell')
def main(atoms, face, vacuum, size, cell, orth):
    """cut surface"""
    out_err.check_cell(atoms, cell)
    o = f"{atoms.filename.split('.')[-2]}_{face[0]}{face[1]}{face[2]}_{size[0]}{size[1]}{size[2]}.cif"

    surface = build.surface(atoms, face, size[2], vacuum=vacuum/2)

    if orth:
        #vector = surface[-2].position - surface[-1].position
        #vector = find_vector(surface)
        surface_cell = surface.cell.cellpar()
        ase.visualize.view(surface)
        gamma = surface_cell[-1]
        if gamma != 90:
            a = np.sin(np.radians(gamma)) * surface_cell[0]
            b = surface_cell[1]
            surface_cell[1] = a
            surface_cell[0] = b
            surface_cell[-1] = 90
            #surface.rotate(vector, np.array([1, 0, 0]))
            surface.rotate(-gamma, 'z')
            surface.set_cell(surface_cell)
            surface.wrap()
        o = f"{atoms.filename.split('.')[-2]}_{face[0]}{face[1]}{face[2]}_{size[0]}{size[1]}{size[2]}_orth.cif"

    super_surface = supercell.supercell(surface, size[0], size[1], 1)

    super_surface.write(o)
    out_err.cell_output(super_surface)
    out_err.path_output(o)


if __name__ == '__main__':
    main()



import MDAnalysis
import numpy as np
from . import numpy_geo


def update_water(self, o_group, h_group, distance_judg=1.2, angle_judg:tuple[float, float]=(None, None), return_index=False):
    """
    input: o and h atom
    output: o and two h in this frame
    """
    oh_pair = MDAnalysis.lib.distances.capped_distance(o_group.positions, h_group.positions, min_cutoff=0, max_cutoff=distance_judg, box=self.u.dimensions, return_distances=False)

    oh_o = oh_pair[:, 0]
    unique_oh_o = np.unique(oh_o)

    group_oh_h = {}
    for oh_o_index in unique_oh_o:
        oh_h_index = oh_pair[oh_o == oh_o_index, 1]
        group_oh_h[oh_o_index] = oh_h_index

    oh1_list = []
    oh2_list = []
    o_list = []
    for oh_o_index in unique_oh_o:
        oh_h_index = group_oh_h[oh_o_index]
        if angle_judg is not None and len(oh_h_index) >= 2:
                for i in range(len(oh_h_index)):
                    for j in range(i + 1, len(oh_h_index)):
                            h1_index = oh_h_index[i]
                            h2_index = oh_h_index[j]

                            o_pos = o_group[oh_o_index].position
                            h1_pos = h_group[h1_index].position
                            h2_pos = h_group[h2_index].position

                            oh1_vec = h1_pos - o_pos
                            oh2_vec = h2_pos - o_pos

                            angle_deg = numpy_geo.vector_vector_angle(oh1_vec, oh2_vec)

                            if angle_judg[0] <= angle_deg <= angle_judg[1]:
                                o_list.append(oh_o_index)
                                oh1_list.append(h1_index)
                                oh2_list.append(h2_index)
        elif len(oh_h_index) == 2:
            o_list.append(oh_o_index)
            oh1_list.append(oh_h_index[0])
            oh2_list.append(oh_h_index[1])
    oh1_index = np.array(oh1_list)
    oh2_index = np.array(oh2_list)
    o_index = np.array(o_list)

    if return_index:
        return o_index, oh1_index, oh2_index
    else:
        if len(o_index) == 0:
            raise ValueError("No water found in this atom group")

        o = o_group[o_index]
        oh1 = h_group[oh1_index]
        oh2 = h_group[oh2_index]
        return o, oh1, oh2



#!/usr/bin/env python3

from ase import build
import os
import click
import numpy as np
import MDAnalysis
from mdkits.util import arg_type, encapsulated_ase, out_err


@click.command(name='adsorbate')
@click.argument('atoms', type=arg_type.Structure)
@click.argument('adsorbate', type=arg_type.Molecule)
@click.option('--cell', type=arg_type.Cell, help='set cell, a list of lattice: --cell x,y,z or x,y,z,a,b,c')
@click.option('--select', type=str, help="select adsorbate position")
@click.option('--height', type=float, help='height above the surface')
@click.option('--rotate', type=click.Tuple([float, float, float]), help='rotate adsorbate molcule around x, y, z axis', default=(0, 0, 0), show_default=True)
@click.option('--offset', type=click.Tuple([float, float]), help='adjust site', default=(0, 0), show_default=True)
@click.option("--cover", type=int, help='cover the surface with adsorbate randomly')
def main(atoms, adsorbate, cell, select, height, rotate, offset, cover):
    """add adsorbate molcule to the surface"""
    if height is None:
        raise ValueError("height is required")

    out_err.check_cell(atoms, cell)
    offset = np.array(offset)
    u = encapsulated_ase.atoms_to_u(atoms)

    molecule = build.molecule(adsorbate)
    molecule.rotate(rotate[0], 'x')
    molecule.rotate(rotate[1], 'y')
    molecule.rotate(rotate[2], 'z')

    output_filename = f"{atoms.filename.split('.')[0]}_{adsorbate}.cif"

    s = u.select_atoms(select)
    positions = s.positions[:, 0:2] + offset
    if cover:
        for index in np.random.choice(positions.shape[0], cover, replace=False):
            build.add_adsorbate(atoms, molecule, height, position=positions[index])
    else:
        for position in positions:
            build.add_adsorbate(atoms, molecule, height, position=position)


    atoms.write(output_filename, format='cif')

    print(os.path.abspath(output_filename))


if __name__ == '__main__':
    main()




import os
import matplotlib.pyplot as plt


def savefig(fig, name, fold=None):
    current_path = os.getcwd()
    if fold:
        if not os.path.exists(fold):
            os.mkdir(fold)
        fig.savefig(f"{current_path}/{fold}/{name}.png", format='png')
        vector_fig_dir = f'pdf'
        if not os.path.exists(vector_fig_dir):
            os.mkdir(vector_fig_dir)
        vector_fig_dir = f'pdf/{fold}'
        if not os.path.exists(vector_fig_dir):
            os.mkdir(vector_fig_dir)
        fig.savefig(f"{current_path}/{vector_fig_dir}/{name}.pdf", format='pdf', transparent=True)
    else:
        fig.savefig(f"{current_path}/{name}.png", format='png')
        vector_fig_dir = f'pdf'
        if not os.path.exists(vector_fig_dir):
            os.mkdir(vector_fig_dir)
        fig.savefig(f"{current_path}/{vector_fig_dir}/{name}.pdf", format='pdf', transparent=True)


def scaled_figsize(scale_w=1.0, scale_h=1.0):
    default_size = plt.rcParams["figure.figsize"]
    return [default_size[0]*scale_w, default_size[1]*scale_h]
